{$N+,E+,F-}
{****************************************************************************
 *           } Unit Calcul;{ (c) 1999 Michal Cihar                          *
 ****************************************************************************
 ****************************************************************************
 * Pravidla pro vzorce:                                                     *
 *  - pri nasobeni je mozno vynechat znamenko (5x=5*x), ale pokud je ve     *
 *    vyrazu x5 tak to znamena promena s nazvem "X5"                        *
 *  - zavorky: - pokud je ve vzorci vice zacatku nez koncu zavorek          *
 *               procedura doplni potrebny pocet koncu zavorek na konec     *
 *               vyrazu                                                     *
 *             - pokud je ve vzorci vice koncu nez zacatku zavorek jedna se *
 *               o chybu                                                    *
 *  - priklady:                                                             *
 *   vyraz    je ekvivaletni s   vyrazem                                    *
 *   5x                          5*x                                        *
 *   5(x+1                       5*(x+1)                                    *
 ****************************************************************************
 * Popis:                                                                   *
 *  Unit Calcul obsahuje funkci pro rekurzivni vyhodnocovani vyrazu         *
 *  zadaneho behem spusteni programu.                                       *
 *                                                                          *
 *  Podporovany jsou operatory (+ - * / \ % ~ ^ ! =), zavorky, funkce,      *
 *  konstanty a promenne. Vetsina funkci je pocitana standartnimi funkcemi  *
 *  obsazenymi v Turbo (Borland) Pascalu krome nekterych funkci, ktere      *
 *  jsou podporovany rozhranim Borlandu ke koprocesoru nebo k emulatoru.    *
 *  Cisla (vcetne promennych) jsou uchovavana v typu extended (rozsah       *
 *  3.4e-4932..1.1e4932 plus znamenko, platnych cislic 19-20, bytu: 10).    *
 *  Pri vypoctech se provadi kontrola preteceni, podteceni kontrolovano     *
 *  neni. Promenne muzou byt ulozeny do souboru na disku.                   *
 *                                                                          *
 *  Vyraz je vyhodnocovan zleva doprava a operatory (funkce) jsou           *
 *  vyhodnocovany v poradi podle priorit:                                   *
 *        1. =     (prirazeni)                                              *
 *        2. funkce                                                         *
 *           5pi,~ (nasobek promenne/konstanty,exponent)                    *
 *        3. (,)   (zavorky)                                                *
 *        4. !     (faktorial)                                              *
 *        5. +,-   (znamenko)                                               *
 *        6. ^     (mocnina)                                                *
 *        7. *,/,% (nasobeni,deleni,celociselne deleni)                     *
 *        8. +,-   (scitani,odcitani)                                       *
 ****************************************************************************
 * Historie:                                                                *
 *  #=informace, -=odstraneno, +=pridano, !=datum, %=zmena                  *
 *  Verze : Zmeny                                                           *
 *    0.1 : # prvni snad funkcni verze                                      *
 *          ! 21.3.1998                                                     *
 *    0.11: # prvni opravdu funkcni verze  ;-)                              *
 *          ! 28.3.1998                                                     *
 *          - odstraneny chyby: ve funkcich, vyhodnocovani, ...             *
 *          - odstranena funkce ZRUS (presunuta do MISO Calc)               *
 *    0.2 : # zmena ve zpracovani nekterych funkci                          *
 *          # zatim to vypada, ze to opravdu funguje                        *
 *          ! 25.11.1998                                                    *
 *          % rozdeleno do vice unit                                        *
 *          + parametry se daji zadavat bez (): sin pi                      *
 *          + vyhodnoceni vyrazu 5(... jako 5*(...                          *
 *          - odstraneny chyby v zaokrouhleni a mocninach                   *
 *          + vysledek minule funkce je uchovavan v konstante ANS           *
 *          + pokud vyraz zacina operaci je pred tuto operaci dosazen       *
 *            vysledek minuleho vypoctu *12 je ANS*12                       *
 *    0.5 : # Zmena vyhodnocovani chyb                                      *
 *          ! 27.11.1998                                                    *
 *          % pokud vyraz zacina operaci +/- neni dosazeno ANS (viz 0.2)    *
 *          % pokud cislo obsahuje vice desetinnych tecek pouzije se prvni  *
 *          + nyni se da zjistit typ chyby                                  *
 *          + pridan operator \ - zbytek po celociselnem deleni             *
 *          - zrusen typ druh operace a souvisejici procedury               *
 *    0.6 : # Pridany funkce                                                *
 *          !  2.12.1998                                                    *
 *          % opravena chyba: cislo staci zadat .9 neni potreba 0.9         *
 *          % funkcedata je promenna                                        *
 *          % pokud vyraz zacina operaci + je dosazeno ANS (viz 0.2 a 0.5)  *
 *          + pridany hyperbolicke a hyperbolometricke funkce               *
 *          + pridan faktorial                                              *
 *          + pridany kombinace a variace                                   *
 *          + v konstantach pribyly predpony jednotek (Exa-yokto)           *
 *    0.7 : # Opraveny chyby                                                *
 *          ! 10.12.1998                                                    *
 *          % opravena chyba: 1neco neni run time error                     *
 *          % opravena chyba: sin neni run time error                       *
 *    1.0: # Opraveny chyby                                                *
 *          ! 16.12.1998                                                    *
 *          % ANS se nepridava v teto unit, ale v MISO Calc                 *
 *          % opravena chyba / neni run time error                          *
 *          % u periodickych fuknci se prepocitava parametr do zakl. periody*
 *          + ruzne jednotky uhlu (deg, rad, grad)                          *
 *          + konverze jednotek uhlu                                        *
 *          - odstranena spousta chyb pri neocekavanem konci vyrazu         *
 *    1.0 : # Opraveny chyby                                                *
 *          # Prvni uvolnena verze                                          *
 *          ! 5.1.1999                                                      *
 *          % odstaneny ruzne malickate chybicky :-)                        *
 *          + pridana kontrola preteceni (snad funkcni)                     *
 *          + pribyly funkce MAX a MIN                                      *
 *    1.1 : # Opraveny chyby, pridany funkce                                *
 *          ! 9.1.1999                                                      *
 *          % odstanena chyba v 2. mocnine/odmocnine - kontrola < 0         *
 *          + pribyly funkce MAXA, MINA, CU, CUR, INV, AVG, GAV a HAV       *
 ****************************************************************************}

{****************************************************************************
 *                             }Interface{                                  *
 ****************************************************************************}
uses Umath,Upamet,Udata,strfunc;
const CalculVerze='1.1';
      CalculDatum='9.1.1999';
      CalculJmeno='Calcul unit (c) 1999 Michal Cihar';

function Vyhodnot(Co : Retezec;var JePlatny:Boolean) : TCislo;

{****************************************************************************
 *                           }implementation{                               *
 ****************************************************************************}
{ted definuju vsechny forward procedury a funkce}
procedure Priorita2(var Vysledek : TCislo) ;    forward;
procedure Priorita5(var Vysledek : TCislo) ;    forward;
procedure Priorita6(var Vysledek : TCislo) ;    forward;
procedure PocitejFunkci(var Vysledek : TCislo); forward;

var
   Pozice  : byte;
    Vyraz : Retezec; {string255}

{****************************************************************************
 *                             PripravChybu                                 *
 ****************************************************************************
 Nastavi globalni promenne na hodnotu aktualni chyby}
procedure PripravChybu(const typ,kde:byte;const info:string);
begin
  chybacislo:=typ;
  ChybaNa:=kde;
  chybastr:=info;
end; {PripravChybu}

{****************************************************************************
 *                             PreskocMezery                                *
 ****************************************************************************
 Preskoci v promenne Vyraz mezeru od pozice Pozice a nastavi Pozice na tuto
 pozici}
procedure PreskocMezery;
begin
     While Vyraz [Pozice] = ' ' do Inc(Pozice);
end; {PreskocMezery}

{****************************************************************************
 *                             CtiPodvyraz                                  *
 ****************************************************************************
 Cte z promenne Vyraz podvyraz oddeleny znakem z promenne Oddelovace}
function CtiPodvyraz : string;
var
   Cteno : string;
begin
     Cteno := '';
     While not(Vyraz [Pozice] in oddelovace)
            and (Pozice <= Length(Vyraz)) do
     begin
          Cteno := Cteno + UpCase(Vyraz[Pozice]);
          Inc(Pozice);
     end;
     CtiPodvyraz := Cteno;
end; {CtiPodvyraz}

{****************************************************************************
 *                             CtiCislo                                     *
 ****************************************************************************
 Cte cislo nebo nasobek konstanty nebo funkce z promenne Vyraz}
function CtiCislo : TCislo;
var
   Cislo,dalsi: TCislo;
   Kod,kod2   : Integer;
   b:boolean;
   Podvyraz : string;
   NeCislo:string;
   op:char;
   Pozice_temp:byte;
begin
  Pozice_temp:=Pozice;
  Podvyraz := CtiPodvyraz;
  if Podvyraz[1] = '.' then Podvyraz := '0' + Podvyraz else
  if Podvyraz[1] = '~' then Podvyraz := '1' + Podvyraz;
  if upcase(Podvyraz[1]) in ['A'..'Z','_'] then
  begin;
     pocitejfunkci(cislo);
  end
  else {neni pismeno ani _}
  begin;
     kod:=nacislo(Podvyraz,Cislo);
     if Kod <> 0 then
     if podvyraz[length(podvyraz)]='~' then
     begin;
        preskocmezery;
        if vyraz[Pozice] in ['+','-'] then
        begin;
           op:=vyraz[Pozice];
           kod:=nacislo(copy(podvyraz,1,kod-1),Cislo);
           inc(Pozice);
           preskocmezery;
           Pozice_temp:=Pozice;
           podvyraz:=ctipodvyraz;
           kod:=nacislo(Podvyraz,Dalsi);
           if kod = 0 then
           begin;
              case op of '+':begin;
                              if testp10(dalsi)and testnas(cislo,power10(dalsi)) then
                              cislo:=cislo*power10(dalsi)
                              else pripravchybu(preteceni,Pozice_temp,'');
                             end;
                         '-':cislo:=cislo*power10(-(dalsi));
                         end;
           end else
           begin;
              Pozice:=Pozice_temp+kod-1;
              Cislo := 0;
              pripravchybu(SpatnyExponent,Pozice,podvyraz);
           end;
        end
        else {neni +/-}
        begin;
           Pozice:=Pozice_temp+kod-1;
           Cislo := 0;
           pripravchybu(ChybiExponent,Pozice,'');
        end;
     end else {posledni neni ~}
     if pos('~',podvyraz)>0 then
     begin;
           kod2:=nacislo(copy(podvyraz,kod+1,length(podvyraz)-kod),Dalsi);
           kod:=nacislo(copy(podvyraz,1,kod-1),Cislo);
           if kod2 = 0 then
                             begin;
                              if testp10(dalsi)and testnas(cislo,power10(dalsi)) then
                              cislo:=cislo*power10(dalsi)
                              else pripravchybu(preteceni,Pozice_temp,'');
                             end
           else
           begin;
              Pozice:=Pozice_temp+kod+kod2-1;
              Cislo := 0;
              pripravchybu(SpatnyExponent,Pozice,podvyraz);
           end;
     end
     else {neni ~}
     if upcase(podvyraz[kod]) in ['A'..'Z','_'] then
        begin;
           nacislo(copy(podvyraz,1,kod-1),Cislo);
           Pozice:=Pozice_temp+kod-1;
           pocitejfunkci(dalsi);
           if chybacislo = 0 then
           cislo:=cislo*dalsi;
     end
     else {neni pismeno ani _}
     begin;
        Pozice:=Pozice_temp+kod-1;
        Cislo := 0;
        pripravchybu(NeniCislo,Pozice,podvyraz);
     end;
  end;
  CtiCislo := Cislo;
end; {CtiCislo}


{****************************************************************************
 *                             ZjistiParamer                                *
 ****************************************************************************
 Cte prametr z promenne Vyraz a pozice Pozice }
procedure ZjistiParametr(var Vysledek : TCislo);
begin
        PreskocMezery;
        if (Vyraz [Pozice] <> '(') then
           begin;
                     priorita5(vysledek);
                     PreskocMezery;
           end
           else begin
                     priorita6(vysledek);
                     PreskocMezery;
           end;
end; {ZjistiParametr}

{****************************************************************************
 *                             ZjistiParamery                               *
 ****************************************************************************
 Cte 2 prametry z promenne Vyraz a pozice Pozice oddelene carkou}
procedure ZjistiParametry(var Vysledek1,Vysledek2 : TCislo);
begin
        ctipodvyraz;
        PreskocMezery;
        if (Vyraz [Pozice] <> '(') then
                        pripravchybu(ChybiZacatekZavorky,Pozice,'')
           else begin
                     Inc(Pozice);
                     Priorita2(Vysledek1);
                     PreskocMezery; {popojdem k ,}
                     if Vyraz[Pozice] <> ',' then
                        pripravchybu(ChybiCarka,Pozice,'') else
                     begin;;
                     Inc(Pozice);
                     PreskocMezery;
                     Priorita2(Vysledek2);
                     PreskocMezery; {popojdem k )}
                     if Vyraz[Pozice] <> ')' then
                        pripravchybu(ChybiKonecZavorky,Pozice,'') else
                     begin;
                     Inc(Pozice);
                     PreskocMezery;
                     end;
                     end;
           end;
end; {ZjistiParametry}

{$F+  musi byt far, protoze pouzivam jako typ procedura
 ****************************************************************************
 *                                Zacatek Funkci                            *
 ****************************************************************************}
(****************************************************************************
 *                                PocitejNahodne                            *
 ****************************************************************************)
procedure PocitejNahodne(var Vysledek:TCislo);
begin
	Vysledek := random;
end; {PocitejNahodne}

(****************************************************************************
 *                                PocitejOrez                               *
 ****************************************************************************)
procedure PocitejOrez(var Vysledek : TCislo);
begin
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     Vysledek := Orez(Vysledek);
end; {PocitejOrez}

(****************************************************************************
 *                                 PocitejFrac                              *
 ****************************************************************************)
procedure PocitejFrac(var Vysledek : TCislo);
begin
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     Vysledek := frac(Vysledek);
end; {Pocitejfrac}

(****************************************************************************
 *                            PocitejZaokrouhli                             *
 ****************************************************************************)
procedure PocitejZaokrouhli(var Vysledek : TCislo);
begin
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     Vysledek := Zaokrouhli(Vysledek);
end; {PocitejZaokrouhli}

(****************************************************************************
 *                             PocitejMocninu                               *
 ****************************************************************************)
procedure PocitejMocninu(var Vysledek : TCislo);
var
   Vysl1,Vysl2:TCislo;
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametry(Vysl1,Vysl2);
     if chybacislo=zadnachyba then
     if (abs(Vysl2-frac(vysl2))=vysl2)or
        (((Vysl2-frac(vysl2))=vysl2)and(Vysl1<>0))or
        ((Vysl2<0)and(Vysl1>0))or
        ((vysl2>0)and(Vysl1>=0))
        then
        begin;
        if testp(Vysl1,Vysl2) then  Vysledek := power(Vysl1,Vysl2)
                              else pripravchybu(preteceni,Pozice_temp,'');
        end
        else
        begin;
         if (Vysl1=0)and(Vysl2<0) then pripravchybu(DeleniNulou,Pozice_temp,'') else
         if (Vysl1<0) then pripravchybu(SpatnaMocnina,Pozice_temp,'');
        end;
end; {PocitejMocninu}

(****************************************************************************
 *                              PocitejMocninu2                             *
 ****************************************************************************)
procedure PocitejMocninu2(var Vysledek : TCislo);
var
   Vysl:TCislo;
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysl);
     if chybacislo=zadnachyba then
     begin;
     if testp2(vysl) then Vysledek := power2(Vysl)
                     else pripravchybu(preteceni,Pozice_temp,'');
     end
end; {PocitejMocninu2}

(****************************************************************************
 *                              PocitejMocninu10                            *
 ****************************************************************************)
procedure PocitejMocninu10(var Vysledek : TCislo);
var
   Vysl:TCislo;
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysl);
     if chybacislo=zadnachyba then
     if testp10(vysl) then Vysledek := power10(Vysl)
                      else pripravchybu(preteceni,Pozice_temp,'');
end; {PocitejMocninu10}

(****************************************************************************
 *                                 PocitejExp                               *
 ****************************************************************************)
procedure PocitejExp(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     if testexp(vysledek) then Vysledek := exp(Vysledek)
                      else pripravchybu(preteceni,Pozice_temp,'');
end; {PocitejExp}

(****************************************************************************
 *                                   PocitejLn                              *
 ****************************************************************************)
procedure PocitejLn(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     if (Vysledek > 0.0) then Vysledek := ln(Vysledek)
        else pripravchybu(SpatnyLog,Pozice_temp,cislo2str(vysledek));
end; {PocitejLn}

(****************************************************************************
 *                                  PocitejLog10                            *
 ****************************************************************************)
procedure PocitejLog10(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     if (Vysledek > 0.0) then Vysledek := log10(Vysledek)
        else pripravchybu(SpatnyLog,Pozice_temp,cislo2str(vysledek));
end; {PocitejLog10}

(****************************************************************************
 *                                 PocitejLog                               *
 ****************************************************************************)
procedure PocitejLog(var Vysledek : TCislo);
var
   Vysl1,Vysl2:TCislo;
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametry(Vysl1,Vysl2);
     if chybacislo=zadnachyba then
     begin;
     if (Vysl1 > 0.0)and(Vysl2 > 0.0)and(Vysl1 <> 1.0) then Vysledek := ln(Vysl2)/ln(Vysl1)
        else
        begin;
        if (Vysl2 <= 0.0) then
           pripravchybu(SpatnyLog,Pozice_temp,cislo2str(vysl2)) else
        if (Vysl1 <= 0.0)or(Vysl1 = 1.0) then
           pripravchybu(SpatnyZaklLog,Pozice_temp,cislo2str(vysl1));
     writeln('000');
        end;
     end;
end; {PocitejLog}

(****************************************************************************
 *                              PocitejMaxAbs                               *
 ****************************************************************************)
procedure PocitejMaxAbs(var Vysledek : TCislo);
var
   Vysl1,Vysl2:TCislo;
begin
     ZjistiParametry(Vysl1,Vysl2);
     if chybacislo=zadnachyba then
     begin;
     if abs(vysl1)>abs(vysl2) then vysledek:=vysl1 else vysledek:=vysl2;
     end;
end; {PocitejMaxAbs}

(****************************************************************************
 *                                 PocitejMax                               *
 ****************************************************************************)
procedure PocitejMax(var Vysledek : TCislo);
var
   Vysl1,Vysl2:TCislo;
begin
     ZjistiParametry(Vysl1,Vysl2);
     if chybacislo=zadnachyba then
     begin;
     if vysl1>vysl2 then vysledek:=vysl1 else vysledek:=vysl2;
     end;
end; {PocitejMax}

(****************************************************************************
 *                              PocitejPrumer                               *
 ****************************************************************************)
procedure PocitejPrumer(var Vysledek : TCislo);
var
   Vysl1,Vysl2:TCislo;
begin
     ZjistiParametry(Vysl1,Vysl2);
     if chybacislo=zadnachyba then
     begin;
     vysledek:=(vysl1+vysl2)/2
     end;
end; {PocitejPrumer}

(****************************************************************************
 *                              PocitejPrumerG                              *
 ****************************************************************************)
procedure PocitejPrumerG(var Vysledek : TCislo);
var
   Vysl1,Vysl2:TCislo;
begin
     ZjistiParametry(Vysl1,Vysl2);
     if chybacislo=zadnachyba then
     begin;
     vysledek:=sqrt(abs(vysl1*vysl2))
     end;
end; {PocitejPrumerG}

(****************************************************************************
 *                              PocitejPrumerH                              *
 ****************************************************************************)
procedure PocitejPrumerH(var Vysledek : TCislo);
var
   Vysl1,Vysl2:TCislo;
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametry(Vysl1,Vysl2);
     if chybacislo=zadnachyba then
     begin;
     if (vysl1<>0)and(vysl2<>0) then vysledek:=2/((1/vysl1)+(1/vysl2))
                                else pripravchybu(deleninulou,Pozice_temp,'');
     end;
end; {PocitejPrumerH}

(****************************************************************************
 *                                 PocitejMin                               *
 ****************************************************************************)
procedure PocitejMin(var Vysledek : TCislo);
var
   Vysl1,Vysl2:TCislo;
begin
     ZjistiParametry(Vysl1,Vysl2);
     if chybacislo=zadnachyba then
     begin;
     if vysl1<vysl2 then vysledek:=vysl1 else vysledek:=vysl2;
     end;
end; {PocitejMin}

(****************************************************************************
 *                              PocitejMinAbs                               *
 ****************************************************************************)
procedure PocitejMinAbs(var Vysledek : TCislo);
var
   Vysl1,Vysl2:TCislo;
begin
     ZjistiParametry(Vysl1,Vysl2);
     if chybacislo=zadnachyba then
     begin;
     if abs(vysl1)<abs(vysl2) then vysledek:=vysl1 else vysledek:=vysl2;
     end;
end; {PocitejMinAbs}

(****************************************************************************
 *                              PocitejLog2                                 *
 ****************************************************************************)
procedure PocitejLog2(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     if (Vysledek > 0.0) then Vysledek := log2(Vysledek)
        else pripravchybu(SpatnyLog,Pozice_temp,cislo2str(vysledek));
end; {PocitejLog2}

(****************************************************************************
 *                             PocitejInverzni                              *
 ****************************************************************************)
procedure PocitejInverzni(var Vysledek : TCislo);
var Pozice_temp:byte;
begin
     Pozice_temp:=pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
     if vysledek<>0 then Vysledek := 1/Vysledek
                    else pripravchybu(deleninulou,Pozice_temp,'');
     end;
end; {PocitejInverzni}

(****************************************************************************
 *                                PocitejAbs                                *
 ****************************************************************************)
procedure PocitejAbs(var Vysledek : TCislo);
begin
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     Vysledek := Abs(Vysledek);
end; {PocitejAbs}

(****************************************************************************
 *                               PocitejSin                                 *
 ****************************************************************************)
procedure PocitejSin(var Vysledek : TCislo);
begin
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
     while narad(vysledek)>(pi_2) do vysledek:=vysledek-zrad(pi_2);
     while narad(vysledek)<(-pi_2) do vysledek:=vysledek+zrad(pi_2);
     Vysledek := sin(narad(Vysledek));
     end;
end; {PocitejSin}

(****************************************************************************
 *                                PocitejCos                                *
 ****************************************************************************)
procedure PocitejCos(var Vysledek : TCislo);
begin
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
     while narad(vysledek)>(pi_2) do vysledek:=vysledek-zrad(pi_2);
     while narad(vysledek)<(-pi_2) do vysledek:=vysledek+zrad(pi_2);
     Vysledek := sin(narad(Vysledek)+(pi_pul));
     end;
end; {PocitejCos}

(****************************************************************************
 *                                PocitejTan                                *
 ****************************************************************************)
procedure PocitejTan(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
     while narad(vysledek)>(pi) do vysledek:=vysledek-zrad(pi);
     while narad(vysledek)<(-pi) do vysledek:=vysledek+zrad(pi);
     if ( cos(narad(Vysledek)) <> 0 ) then
	Vysledek := tan(narad(vysledek))
     else pripravchybu(SpatnyTan,Pozice_temp,cislo2str(vysledek));
     end;
end; {PocitejTan}

(****************************************************************************
 *                                PocitejCoTan                              *
 ****************************************************************************)
procedure PocitejCoTan(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
     while narad(vysledek)>(pi) do vysledek:=vysledek-zrad(pi);
     while narad(vysledek)<(-pi) do vysledek:=vysledek+zrad(pi);
     if ( sin(narad(Vysledek)) <> 0 ) then
	Vysledek := cos(narad(Vysledek)) / sin(narad(Vysledek))
     else pripravchybu(SpatnyCoTan,Pozice_temp,cislo2str(vysledek));
     end;
end; {PocitejCoTan}

(****************************************************************************
 *                             PocitejArcSin                                *
 ****************************************************************************)
procedure PocitejArcSin(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
     if (abs(Vysledek) < 1.0) then
	Vysledek := zrad(arcTan(Vysledek/sqrt(1-Vysledek*Vysledek)))
        else
        if vysledek=1.0 then
        vysledek:=zrad(pi_pul)
        else
        if vysledek=-1.0 then
        vysledek:=zrad(-pi_pul)
     else pripravchybu(SpatnyArcSin,Pozice_temp,cislo2str(vysledek));
     end;
end; {PocitejArcSin}

(****************************************************************************
 *                               PocitejArcCos                              *
 ****************************************************************************)
procedure PocitejArcCos(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
	if ((Vysledek <> 0.0) and (abs(Vysledek) < 1.0)) then
	 Vysledek := zrad(arcTan(sqrt(1-Vysledek*Vysledek)/Vysledek))
         else if Vysledek = 0.0 then vysledek:=zrad(pi_pul)
         else if vysledek=1.0 then vysledek:=zrad(0.0)
         else if vysledek=(-1.0) then vysledek:=zrad(-(pi))
         else pripravchybu(SpatnyArcCos,Pozice_temp,cislo2str(vysledek));
     end;
end; {PocitejArcCos}

(****************************************************************************
 *                               PocitejArcTan                              *
 ****************************************************************************)
procedure PocitejArcTan(var Vysledek : TCislo);
begin
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     Vysledek := zrad(ArcTan(Vysledek));
end; {PocitejArcTan}

(****************************************************************************
 *                              PocitejArcCoTan                             *
 ****************************************************************************)
procedure PocitejArcCoTan(var Vysledek : TCislo);
begin
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     Vysledek := zrad((pi_pul)-ArcTan(Vysledek));
end; {PocitejArcTan}

(****************************************************************************
 *                               PocitejSinH                                *
 ****************************************************************************)
procedure PocitejSinh(var Vysledek : TCislo);
var Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
     if abs(vysledek)<=11356 then Vysledek := (1/2)*(exp(Vysledek)-exp(-Vysledek))
                            else pripravchybu(preteceni,Pozice_temp,'');
     end;
end; {PocitejSin}

(****************************************************************************
 *                                PocitejCosH                               *
 ****************************************************************************)
procedure PocitejCosh(var Vysledek : TCislo);
var Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
     if abs(vysledek)<=11356 then Vysledek := (1/2)*(exp(Vysledek)+exp(-Vysledek))
                            else pripravchybu(preteceni,Pozice_temp,'');
     end;
end; {PocitejCos}

(****************************************************************************
 *                                PocitejTanH                               *
 ****************************************************************************)
procedure PocitejTanh(var Vysledek : TCislo);
begin
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     	Vysledek := (exp(Vysledek)-exp(-Vysledek))/(exp(Vysledek)+exp(-Vysledek));
end; {PocitejTan}

(****************************************************************************
 *                                PocitejCoTanH                             *
 ****************************************************************************)
procedure PocitejCoTanh(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     if ( Vysledek <> 0 )and((exp(Vysledek)-exp(-Vysledek))<>0) then
	Vysledek := (exp(Vysledek)+exp(-Vysledek))/(exp(Vysledek)-exp(-Vysledek))
        else pripravchybu(SpatnyCoTanH,Pozice_temp,cislo2str(vysledek));
end; {PocitejCoTan}

(****************************************************************************
 *                             PocitejArgSinH                               *
 ****************************************************************************)
procedure PocitejArgSinh(var Vysledek : TCislo);
begin
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     vysledek:=ln(vysledek+sqrt((vysledek*vysledek)+1));
end; {PocitejArcSin}

(****************************************************************************
 *                               PocitejArgCosH                             *
 ****************************************************************************)
procedure PocitejArgCosh(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
	if (Vysledek >= 1.0) then
           vysledek:=ln(vysledek+sqrt((vysledek*vysledek)-1))
         else pripravchybu(SpatnyArgCosh,Pozice_temp,cislo2str(vysledek));
     end;
end; {PocitejArcCos}

(****************************************************************************
 *                               PocitejArgTanH                             *
 ****************************************************************************)
procedure PocitejArgTanh(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
	if (abs(Vysledek) < 1.0) then
           vysledek:=ln(sqrt( (vysledek+1)/(1-vysledek) ))
         else pripravchybu(SpatnyArgTanh,Pozice_temp,cislo2str(vysledek));
     end;
end; {PocitejArcTan}

(****************************************************************************
 *                              PocitejArgCoTanH                            *
 ****************************************************************************)
procedure PocitejArgCoTanh(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
	if (abs(Vysledek) > 1.0) then
           vysledek:=ln(sqrt( (vysledek+1)/(vysledek-1) ))
         else pripravchybu(SpatnyArgCoTanh,Pozice_temp,cislo2str(vysledek));
     end;
end; {PocitejArcTan}

(****************************************************************************
 *                              Pocitej2Mocninu                             *
 ****************************************************************************)
procedure Pocitej2Mocninu(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
     if testsqr(Vysledek) then Vysledek := Sqr(Vysledek)
                           else pripravchybu(preteceni,Pozice_temp,'');
     end;
end; {Pocitej2Mocninu}

(****************************************************************************
 *                             Pocitej2Odmocninu                            *
 ****************************************************************************)
procedure Pocitej2Odmocninu(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
     if vysledek>=0 then Vysledek := Sqrt(Vysledek)
                    else pripravchybu(SpatnaOdmoc,Pozice_temp,cislo2str(vysledek));
     end;
end; {Pocitej2Odmocninu}

(****************************************************************************
 *                              Pocitej3Mocninu                             *
 ****************************************************************************)
procedure Pocitej3Mocninu(var Vysledek : TCislo);
var
   Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     if testnas(Vysledek,Vysledek) then
     begin;
     if testnas(Vysledek*Vysledek,Vysledek) then Vysledek := Vysledek*Vysledek*Vysledek
                                            else pripravchybu(preteceni,Pozice_temp,'');
     end
                           else pripravchybu(preteceni,Pozice_temp,'');
end; {Pocitej3Mocninu}

(****************************************************************************
 *                             Pocitej3Odmocninu                            *
 ****************************************************************************)
procedure Pocitej3Odmocninu(var Vysledek : TCislo);
begin
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     begin;
     if vysledek<0 then Vysledek := -Power(abs(Vysledek),1/3)
                   else Vysledek := Power(Vysledek,1/3);
     end;
end; {Pocitej3Odmocninu}

(****************************************************************************
 *                             PocitejKombinace                             *
 ****************************************************************************)
procedure PocitejKombinace(var Vysledek : TCislo);
var vysl1,vysl2:tcislo;
    Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametry(Vysl1,vysl2);
     if chybacislo=zadnachyba then
     if vysl1<=vysl2 then
     begin;
     if testfac(Vysl2) then Vysledek := fact(Vysl2)/(fact(Vysl1)*fact(vysl2-Vysl1))
                       else pripravchybu(preteceni,Pozice_temp,'');
     end
                     else pripravchybu(SpatnYKombinace,Pozice_temp,cislo2str(vysl1));
end; {PocitejKombinace}

(****************************************************************************
 *                             PocitejVariace                               *
 ****************************************************************************)
procedure PocitejVariace(var Vysledek : TCislo);
var vysl1,vysl2:tcislo;
    Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametry(Vysl1,vysl2);
     if chybacislo=zadnachyba then
     if vysl1<=vysl2 then
     begin;
     if testfac(Vysl2) then Vysledek := fact(Vysl2)/(fact(vysl2-Vysl1))
                       else pripravchybu(preteceni,Pozice_temp,'');
     end
                     else pripravchybu(SpatnyVariace,Pozice_temp,cislo2str(vysl1));
end; {PocitejVariace}

(****************************************************************************
 *                             PocitejRad                                   *
 ****************************************************************************)
procedure PocitejRad(var Vysledek : TCislo);
var Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     case UhlovaMira of deg:if testnas(Vysledek,180/pi) then Vysledek:=(Vysledek/pi)*180
                                                         else pripravchybu(preteceni,Pozice_temp,'');
                        rad:Vysledek:=Vysledek;
                        grad:if testnas(Vysledek,200/pi) then Vysledek:=(Vysledek/pi)*200
                                                           else pripravchybu(preteceni,Pozice_temp,'');
                        end;
end; {PocitejRad}

(****************************************************************************
 *                             PocitejGrad                                  *
 ****************************************************************************)
procedure PocitejGrad(var Vysledek : TCislo);
var Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     case UhlovaMira of deg:if testnas(Vysledek,9/10) then Vysledek:=(Vysledek/10)*9
                                                       else pripravchybu(preteceni,Pozice_temp,'');
                        rad:if testnas(Vysledek,pi/200) then Vysledek:=(Vysledek/200)*pi
                                                        else pripravchybu(preteceni,Pozice_temp,'');
                        grad:Vysledek:=Vysledek;
                        end;
end; {PocitejGrad}

(****************************************************************************
 *                             PocitejDeg                                   *
 ****************************************************************************)
procedure PocitejDeg(var Vysledek : TCislo);
var Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     ZjistiParametr(Vysledek);
     if chybacislo=zadnachyba then
     case UhlovaMira of deg:Vysledek:=Vysledek;
                        rad:if testnas(Vysledek,180/pi) then Vysledek:=(Vysledek/180)*pi
                                                        else pripravchybu(preteceni,Pozice_temp,'');
                        grad:if testnas(Vysledek,9/10) then Vysledek:=(Vysledek/9)*10
                                                       else pripravchybu(preteceni,Pozice_temp,'');
                        end;
end; {PocitejDeg}
{$F-  musi byt far, protoze pouzivam jako typ procedura
 ****************************************************************************
 *                                Konec Funkci                              *
 ****************************************************************************}

{****************************************************************************
 *                             PocitejFunkci                                *
 ****************************************************************************
 Zjisti hodnotu aktualniho vyrazu - funkce,promenne,konstanty}
procedure PocitejFunkci(var Vysledek : TCislo);
var cislo:byte;
    Pocitano,b:boolean;
    Pozice_temp:byte;
    zjist:string;
begin
Pocitano:=false;
for cislo:=1 to maxfunkci do
   if navelka(bezoddelovacu(copy(vyraz,Pozice,10)))=FunkceData[cislo].jmeno then
      begin;
            Pozice:=Pozice+length(FunkceData[cislo].jmeno);
            FunkceData[cislo].Proved(Vysledek);
            pocitano:=true;
            break;
      end;
if not pocitano then
   begin;
     Pozice_temp:=Pozice;
     zjist:=ctipodvyraz;
     if je_promenna(zjist) then Vysledek:=z_pameti(zjist,b) else
     if je_konst(zjist) then Vysledek:=z_konst(zjist,b) else
                        pripravchybu(SpatneJmeno,Pozice_temp,zjist);
   end;
end;

{****************************************************************************
 *                             Zakladni                                     *
 ****************************************************************************
 Zjisti jestli je prvni znak cislo nebo pismeno a podle toho spusti funkci}
procedure Zakladni(var Vysledek : TCislo);
begin
     if chybacislo=zadnachyba then begin;
     if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'');
     case upcase(Vyraz [Pozice]) of '0'..'9','.','~':Vysledek := CtiCislo;
                               'A'..'Z','_':PocitejFunkci(Vysledek);
                               else
                               begin;
                               if vyraz[Pozice] in oddelovace then
                               pripravchybu(ChybiHodnota,Pozice,ctipodvyraz)
                               else
                               pripravchybu(spatnejmeno,Pozice,ctipodvyraz);
                               end;
                               end;
     PreskocMezery;
     end;
end;

{****************************************************************************
 *                             Priorita7                                    *
 ****************************************************************************
 Zavorky}
procedure Priorita7(var Vysledek : TCislo);
begin
     if (Pozice<=length(vyraz)) then
     if (Vyraz [Pozice] = '(') then begin
        Inc(Pozice);
        preskocmezery;
        if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'');
        Priorita2(Vysledek);
        PreskocMezery; {dojki k )}
        if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'');
        if Vyraz[Pozice] <> ')' then pripravchybu(Chybikoneczavorky,Pozice,'');
        Inc(Pozice);
        PreskocMezery;
     end else Zakladni (Vysledek);
end;

{****************************************************************************
 *                                 Priorita6                                *
 ****************************************************************************
 Faktorial (!) do 1755}
procedure Priorita6(var Vysledek : TCislo);
var Pozice_temp : byte;
begin
     Pozice_temp:=Pozice;
     preskocmezery;
     if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'');
     Priorita7(Vysledek);
     if chybacislo=zadnachyba then begin
     preskocmezery;
     if (Pozice<=length(vyraz)) then
     if (vysledek<0)and(Vyraz [Pozice]= '!') then pripravchybu(SpatnyFaktorial,Pozice_temp,cislo2str(vysledek))
     else
     while (Vyraz [Pozice]= '!')and(Pozice<=length(vyraz))and(chybacislo=zadnachyba) do
      if testfac(vysledek) then
        begin;
          Vysledek := fact(Vysledek);
          Inc(Pozice);
          preskocmezery;
        end
        else pripravchybu(preteceni,Pozice_temp,'');
     end;
end;

{****************************************************************************
 *                                 Priorita5                                *
 ****************************************************************************
 Znamenko vyrazu (unarni +,-)}
procedure Priorita5(var Vysledek : TCislo);
var op : Char;
begin
     op := Vyraz [Pozice];
     if op in ['-','+'] then Inc(Pozice);
     preskocmezery;
     if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'');
     Priorita6(Vysledek);
     if op = '-' then Vysledek := - Vysledek;
end;

{****************************************************************************
 *                                 Priorita4                                *
 ****************************************************************************
 mocnina}
procedure Priorita4(var Vysledek : TCislo);
var Dalsi : TCislo;
    Pozice_temp:byte;
begin
     Pozice_temp:=Pozice;
     preskocmezery;
     if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'');
     Priorita5(Vysledek);
     if chybacislo=zadnachyba then
        if (Pozice<=length(vyraz)) then
        if Vyraz [Pozice] = '^' then
        begin
           Inc(Pozice);
           preskocmezery;
           if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'') else
           begin;
           Priorita4(Dalsi);
           if (abs(Dalsi-frac(Dalsi))=Dalsi)or
              (((Dalsi-frac(Dalsi))=Dalsi)and(Vysledek<>0))or
              ((Dalsi<0)and(Vysledek>0))or
              ((Dalsi>0)and(Vysledek>=0))
              then
              begin;
                if testp(Vysledek,Dalsi) then  Vysledek := power(Vysledek,Dalsi)
                                         else pripravchybu(preteceni,Pozice_temp,'');
              end
           else
              begin;
               if (Vysledek=0)and(Dalsi<0) then pripravchybu(DeleniNulou,Pozice_temp,'') else
               if (Vysledek<0) then pripravchybu(SpatnaMocnina,Pozice_temp,'');
              end;
           end;
        end;
end;

{****************************************************************************
 *                             Priorita3                                    *
 ****************************************************************************
 nasobeni, deleni, celociselne deleni(%), zbytek po deleni(\)}
procedure Priorita3(var Vysledek : TCislo);
var Dalsi : TCislo;
    op   : Char;
    Pozice_temp:byte;
begin
     preskocmezery;
     if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'');
     Priorita4(Vysledek);
     if chybacislo=zadnachyba then begin
        PreskocMezery;
        While (Vyraz [Pozice] in ['*','/','%','\','('])and(Pozice<=length(vyraz))and(chybacislo=zadnachyba) do
        begin
         op := Vyraz[Pozice];
         if op<>'(' then Inc(Pozice);
         if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'') else
         begin;
         if op='(' then
         begin;
               preskocmezery;
               if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'');
               priorita6(Dalsi);
               if chybacislo=zadnachyba then
               vysledek:=vysledek*dalsi;
         end else
         begin;
           Pozice_temp:=Pozice;
           preskocmezery;
           if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'');
           Priorita4(Dalsi);
           if chybacislo=zadnachyba then begin
           if op = '*' then
           begin;
           if testnas(vysledek,dalsi) then Vysledek := Vysledek * Dalsi
           else pripravchybu(preteceni,Pozice_temp,'');
           end
              else begin
	        if (Dalsi = 0.0) then pripravchybu(Deleninulou,Pozice_temp,'')
	      	else
                begin;
                if testnas(vysledek,1/dalsi) then
                    case op of '/':Vysledek := Vysledek / Dalsi;
                               '%':Vysledek := vydel(Vysledek,Dalsi);
                               '\':Vysledek := zbytek(Vysledek,Dalsi);
                               end
                else pripravchybu(preteceni,Pozice_temp,'');
                end;
	      end;
           PreskocMezery;
           end;
           end; {if Pozice>...}
         end;
        end; {while}
     end; {neni chyba}
end;

{****************************************************************************
 *                             Priorita2                                    *
 ****************************************************************************
 scitani, odcitani}
procedure Priorita2(var Vysledek : TCislo);
var Dalsi : TCislo;
    op   : Char;
    Pozice_temp:byte;
begin
     preskocmezery;
     if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'');
     Priorita3(Vysledek);
     if chybacislo=zadnachyba then begin
        PreskocMezery;
        While (Vyraz [Pozice] in ['+','-'])and(Pozice<=length(vyraz))and(chybacislo=zadnachyba) do begin
         op := Vyraz [Pozice];
         inc(Pozice);
         if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'')
         else
         begin;
           preskocmezery;
           if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'');
           Pozice_temp:=Pozice;
           Priorita3(Dalsi);
           if chybacislo=zadnachyba then
           begin;
           if op = '+' then
           begin;
            if testscit(vysledek,dalsi) then Vysledek := Vysledek + Dalsi
            else pripravchybu(preteceni,Pozice_temp,'');
           end else
           begin;
            if testscit(vysledek,-dalsi) then Vysledek := Vysledek - Dalsi
            else pripravchybu(preteceni,Pozice_temp,'');
           end;
           PreskocMezery;
           end; {chyba}
         end;
        end; {while}
     end; {neni chyba}
end;

{****************************************************************************
 *                             Priorita1                                    *
 ****************************************************************************
 prirazeni}
procedure Priorita1(var Vysledek : TCislo);
var temp : string;
begin
     if pos('=',vyraz)>0 then
     begin;
         temp:=ctipodvyraz;
         Pozice:=pos('=',vyraz)+1;
         preskocmezery;
         if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'');
         priorita2(Vysledek);
         if chybacislo=zadnachyba then do_pameti(temp,vysledek);
         if chybapameti_<>0 then pripravchybu(ChybaVPameti,1,temp);
     end {je =}
     else
     begin;
       preskocmezery;
       if (Pozice>length(vyraz)) then pripravchybu(NeocekavanyKonec,Pozice,'');
       Priorita2(Vysledek);
     end;
end;

{****************************************************************************
 *                               Vyhodnot                                   *
 ****************************************************************************}
function Vyhodnot(Co : Retezec;var JePlatny:Boolean) : TCislo;
var Vysledek : TCislo;
begin
     Pozice := 1;
     vyraz:=co;
     chybacislo:=0;
     ChybaNa:=0;
     chybastr:='';
     {opravime pocet zavorek:}
     if pocetznaku('(',vyraz)>pocetznaku(')',vyraz) then
        vyraz:=vyraz+vratnznaku(')',pocetznaku('(',vyraz)-pocetznaku(')',vyraz));
     preskocmezery;
     Vysledek := 0; {kdyz nic nespocitam}
     if (Pozice<=length(vyraz))and(not(vyraz[Pozice] in [';'])) then Priorita1(Vysledek);
     Vyhodnot := Vysledek;
     zmen_ans(vysledek);
     if chybacislo<>zadnachyba then JePlatny := False
                               else JePlatny := True;
end;

{****************************************************************************
 *                                    MAIN                                  *
 ****************************************************************************
 inicializace funkci}
begin
with funkcedata[1] do
begin;
{1}           Jmeno:='COS'     ;Proved:=PocitejCos;
end;
with funkcedata[2] do
begin;
               Jmeno:='SIN'     ;Proved:=PocitejSin;
end;
with funkcedata[3] do
begin;
               Jmeno:='TG'      ;Proved:=PocitejTan;
end;
with funkcedata[4] do
begin;
               Jmeno:='COTG'    ;Proved:=PocitejCoTan;
end;
with funkcedata[5] do
begin;
{5}            Jmeno:='EXP'     ;Proved:=PocitejExp;
end;
with funkcedata[6] do
begin;
               Jmeno:='LN'      ;Proved:=PocitejLn;
end;
with funkcedata[7] do
begin;
               Jmeno:='LOG10'   ;Proved:=PocitejLog10;
end;
with funkcedata[8] do
begin;
               Jmeno:='LOG2'    ;Proved:=PocitejLog2;
end;
with funkcedata[9] do
begin;
               Jmeno:='LOG'     ;Proved:=PocitejLog;
end;
with funkcedata[10] do
begin;
{10}           Jmeno:='ARCCOS'  ;Proved:=PocitejArcCos;
end;
with funkcedata[11] do
begin;
               Jmeno:='ARCSIN'  ;Proved:=PocitejArcSin;
end;
with funkcedata[12] do
begin;
               Jmeno:='ARCTG'   ;Proved:=PocitejArcTan;
end;
with funkcedata[13] do
begin;
               Jmeno:='ARCCOTG' ;Proved:=PocitejArcCoTan;
end;
with funkcedata[14] do
begin;
               Jmeno:='SQRT'    ;Proved:=Pocitej2Odmocninu;
end;
with funkcedata[15] do
begin;
{15}           Jmeno:='SQR'     ;Proved:=Pocitej2Mocninu;
end;
with funkcedata[16] do
begin;
               Jmeno:='RANDOM'  ;Proved:=PocitejNahodne;
end;
with funkcedata[17] do
begin;
               Jmeno:='TRUNC'   ;Proved:=PocitejOrez;
end;
with funkcedata[18] do
begin;
               Jmeno:='ROUND'   ;Proved:=PocitejZaokrouhli;
end;
with funkcedata[19] do
begin;
               Jmeno:='ABS'     ;Proved:=PocitejAbs;
end;
with funkcedata[20] do
begin;
{20}           Jmeno:='POWER'   ;Proved:=PocitejMocninu;
end;
with funkcedata[21] do
begin;
               Jmeno:='POWER2'  ;Proved:=PocitejMocninu2;
end;
with funkcedata[22] do
begin;
               Jmeno:='POWER10' ;Proved:=PocitejMocninu10;
end;
with funkcedata[23] do
begin;
               Jmeno:='FRAC'    ;Proved:=PocitejFrac;
end;
with funkcedata[24] do
begin;
               Jmeno:='COSH'    ;Proved:=PocitejCosH;
end;
with funkcedata[25] do
begin;
{25}           Jmeno:='SINH'    ;Proved:=PocitejSinH;
end;
with funkcedata[26] do
begin;
               Jmeno:='TGH'     ;Proved:=PocitejTanH;
end;
with funkcedata[27] do
begin;
               Jmeno:='COTGH'   ;Proved:=PocitejCoTanH;
end;
with funkcedata[28] do
begin;
               Jmeno:='ARGCOSH' ;Proved:=PocitejArgCosH;
end;
with funkcedata[29] do
begin;
               Jmeno:='ARGSINH' ;Proved:=PocitejArgSinH;
end;
with funkcedata[30] do
begin;
{30}           Jmeno:='ARGTGH'  ;Proved:=PocitejArgTanH;
end;
with funkcedata[31] do
begin;
               Jmeno:='ARGCOTGH';Proved:=PocitejArgCoTanH;
end;
with funkcedata[32] do
begin;
               Jmeno:='COMB';Proved:=PocitejKombinace;
end;
with funkcedata[33] do
begin;
               Jmeno:='VAR';Proved:=PocitejVariace;
end;
with funkcedata[34] do
begin;
               Jmeno:='RAD';Proved:=PocitejRad;
end;
with funkcedata[35] do
begin;
               Jmeno:='DEG';Proved:=PocitejDeg;
end;
with funkcedata[36] do
begin;
               Jmeno:='GRAD';Proved:=PocitejGrad;
end;
with funkcedata[37] do
begin;
               Jmeno:='MAX';Proved:=PocitejMax;
end;
with funkcedata[38] do
begin;
               Jmeno:='MIN';Proved:=PocitejMin;
end;
with funkcedata[39] do
begin;
               Jmeno:='CU';Proved:=Pocitej3Mocninu;
end;
with funkcedata[40] do
begin;
               Jmeno:='CUR';Proved:=Pocitej3Odmocninu;
end;
with funkcedata[41] do
begin;
               Jmeno:='INV';Proved:=PocitejInverzni;
end;
with funkcedata[42] do
begin;
               Jmeno:='MAXA';Proved:=PocitejMaxAbs;
end;
with funkcedata[43] do
begin;
               Jmeno:='AVG';Proved:=PocitejPrumer;
end;
with funkcedata[44] do
begin;
               Jmeno:='GAV';Proved:=PocitejPrumerG;
end;
with funkcedata[45] do
begin;
               Jmeno:='HAV';Proved:=PocitejPrumerH;
end;
with funkcedata[46] do
begin;
               Jmeno:='MINA';Proved:=PocitejMinAbs;
end;
end.
